/*
    Copyright 2004,2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>



#define VER         "0.2"
#define BUFFSZ      8192
#define V1          "__VER__1.5.0.8"
#define V2          "__VER__1.8.7.2"
#define B64DECX     bb = *bf; \
                    for(; \
                    bb && \
                    ((bb == ' ') || \
                    (bb == '\t') || \
                    (bb == '\r') || \
                    (bb == '\n')); \
                    bf++);



void argoftp_pwd(unsigned char *pwd, int ver);
unsigned char *Base64Decode(unsigned char *buf, unsigned char *ret);
void std_err(void);



int main(int argc, char *argv[]) {
    FILE            *fd;
    int             ver = 0;
    unsigned char   buff[BUFFSZ + 1];


    setbuf(stdout, NULL);

    fputs("\n"
        "ArGoSoft Mail Server password decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\nUsage: %s <USERDATA.REC>\n"
            "\n"
            "The file userdata.rec is located in the users folders in the application's\n"
            "folder\n"
            "\n", argv[0]);
        fputs("\n   Press RETURN to exit\n", stdout);
        fgetc(stdin);
        exit(1);
    }

    printf("- open file \"%s\"\n", argv[1]);
    fd = fopen(argv[1], "rt");
    if(!fd) std_err();

    fgets(buff, BUFFSZ, fd);
    printf("- version: %s", buff);
    if(!memcmp(buff, V1, sizeof(V1) - 1)) {
        ver = 0;
    } else if(!memcmp(buff, V2, sizeof(V2) - 1)) {
        ver = 1;
    } else {
        fputs("- ALERT: the file uses an unknown version, I try to set the 1.8.7.2 decoding\n", stdout);
        ver = 1;
    }

    fgets(buff, BUFFSZ, fd);
    argoftp_pwd(buff, ver);
    printf("\n  Password:   %s\n", buff);

    fclose(fd);

    fputs("\n\n   Press RETURN to exit\n", stdout);
    fgetc(stdin);
    return(0);
}



void argoftp_pwd(unsigned char *pwd, int ver) {
    int             len;
    unsigned char   key[] =
        "T3ZlciB0aGUgaGlsbHMgYW5kIGZhciBhd2F5LCBUZWxldHViYmllcyBjb21lIHRvIHBsYXk=",
                    *pk;

    len = Base64Decode(pwd, pwd) - pwd;

    if(ver) {
        pk = key;
        while(len--) {
            *pwd++ ^= *pk++;
            if(!*pk) pk = key;
        }
        *pk = 0;
    }
}



unsigned char *Base64Decode(unsigned char *buf, unsigned char *ret) {
    int             c,
                    d,
                    i,
                    *Base;
    unsigned char   bb,
                    *bf,
                    *rf;

    if(!*buf) {
        *ret = 0;
        return(ret);
    }

    Base = (int *)malloc(sizeof(int) << 8);
    if(!Base) {
        strcpy(ret, buf);
        return(ret);
    }
    for(i = 0; i < 26; i++) {
        Base['A' + i] = i;
        Base['a' + i] = i + 26;
    }
    for(i = 0; i < 10; i++) {
        Base['0' + i] = i + 52;
    }
    Base['+'] = 62;
    Base['/'] = 63;

    bf = buf;
    rf = ret;
    B64DECX;
    while(1) {
        bb = *bf++;
        if((bb == '=') || !bb) break;
        c = Base[bb];

        B64DECX;
        bb = *bf++;
        if((bb == '=') || !bb) break;
        d = Base[bb];
        *rf++ = (c << 2) | (d >> 4);

        B64DECX;
        bb = *bf++;
        if((bb == '=') || !bb) break;
        c = Base[bb];
        *rf++ = ((d & 0xF) << 4) | (c >> 2);

        B64DECX;
        bb = *bf++;
        if((bb == '=') || !bb) break;
        *rf++ = ((c & 0x3) << 6) | Base[bb];
    }
    *rf = 0;
    free(Base);
    return(rf);
}



void std_err(void) {
    perror("\nError");
    fputs("\n    Press RETURN to exit\n", stdout);
    fgetc(stdin);
    exit(1);
}

